// ----------------------------------------------------------------------------
//
// umFish20Ex(umFish20Ex.PAS)                             Ulrich Mller 1.03.02
//                                                        Procida 400NT Delphi2
//                                                        Version 2.1
//
// Funktionen zum Betrieb der fischertechnik Interfaces
// auf Basis von umFish20.DLL v2.0 :
//
// Laufzeitdaten       : ftiDCB     : record, kein Aufrufparameter
//                       ftiNotHalt : Abbruchwunsch
//                       nur ein Interface pro Programm ( & Slave)
// Unterbrechbarkeit   : Integration von  Application.ProcessMessages.
// Abbrechbarkeit      : Variable ftiNotHalt, die Escape-Taste und Timelimit
//                       (bei langlaufenden Funktionen)
// Refresh             : durch Pollen von umFish20.DLL gewhrleistet
// Kontrolle Interface : ftiOpenInterface / ftiCloseInterface
//                       (WinNT|parallel : ftiStartDriver / ftiStopDriver)
// Lesen Eingabe       : ftiGetInputs / ftiGetInput, ftiGetAnalog
// Schreiben Ausgabe   : ftiSetMotors / ftiSetMotor / ftiClearMotors,
//                       ftiSetLamp
// Warten auf Ereignis : ftiPause/ftiWaitForTime, ftiWaitForInput,
//                       ftiWaitForChange,
//                       WaitForPositionUp, WaitForPositionDown
//                       ftiWaitForLow, WaitForHigh
// Abbruchwunsch       : ftiFinish
//
// LongInt Parameter um nach umFish20.DLL durchreichen zu knnen, dort ist
// intern DWORD erforderlich. Mgliches LongWord wegen Delphi2 Kompatibilitt
// vermieden ebenso die sinnvoll mglichen Standardwerte bei Wait... und Finish
//
// ----------------------------------------------------------------------------

unit umFish20Ex;

interface
const
  ftiAus=0; ftiEin=1; ftiLinks=1; ftiRechts=2;
  ftiOff=ftiAus; ftiOn=ftiEin; ftiLeft=ftiLinks; ftiRight=ftiRechts;
  ftiFehler=$0001FFFF; ftiError=ftiFehler;
  ftiAEX=0; ftiAEY=1; ftiEx=ftiAEX; ftiEY=ftiAEY;
  ftiE1=1; ftiE2=2; ftiE3=3; ftiE4=4; ftiE5=5; ftiE6=6; ftiE7=7; ftiE8=8;
  ftiE9=9;   ftiE10=10; ftiE11=11; ftiE12=12;
  ftiE13=13; ftiE14=14; ftiE15=15; ftiE16=16;
  ftiM1=1; ftiM2=2; ftiM3=3; ftiM4=4; ftiM5=5; ftiM6=6; ftiM7=7; ftiM8=8;

type TftiDCB = record
  hCom:         LongInt;         // --- PortHandle / Openflag ( = 0)
  PortID:       LongInt;         //     IO-Adresse bei LPT
  LPTDelay:     LongInt;         //     LPT Verzgerungsfaktor
  LPTAnalog:    LongInt;         //     LPT Analogfaktor
  ECount:       LongInt;         //     Anzahl E-Eingnge (8/16)

  FID:          LongInt;         // --- MultiMediaTimer : Handle
  PollInterval: LongInt;         //     Pollinterval des MM-Timers
  AnalogScan:   LongInt;         //     Pollen auuch der Analogeingnge (0/1)

  OutputStatus: LongInt;         // --- Status der M-Ausgnge
  InputStatus:  LongInt;         //     Status der E-Eingnge
  Analogs:      array[0..1] of LongInt; //  Werte der A-Eingnge
  Counters:     array[1..16] of LongInt; // Impulse an den E-Eingngen
  end;
type TpftiDCB = ^TftiDCB;

var
  ftiDCB:     TftiDCB;
  ftiNotHalt: Boolean;

function  ftiOpenInterface(PortName: string): LongInt;
function  ftiCloseInterface: LongInt;
function  ftiStartDriver:LongInt; stdcall; external 'umFish20.dll' name 'StartDriver';
function  ftiStopDriver:LongInt;  stdcall; external 'umFish20.dll' name 'StopDriver';

function  ftiGetInputs: LongInt;
function  ftiGetInput(InputNr: LongInt): Boolean;
function  ftiGetAnalog(AnalogNr: LongInt): LongInt;
function  ftiSetMotors(MotorStatus: LongInt): LongInt;
function  ftiSetMotor(MotorNr, Direction: LongInt): LongInt;
function  ftiClearMotors: LongInt;
function  ftiSetLamp(LampNr, OnOff: LongInt): LongInt;

function  ftiWaitForInput(InputNr: LongInt; OnOff: Boolean): LongInt;
function  ftiWaitForChange(InputNr, NrOfChanges, TermInputNr: LongInt): LongInt;
function  ftiWaitForHigh(InputNr: LongInt): LongInt;
function  ftiWaitForLow(InputNr: LongInt): LongInt;
function  ftiWaitForPositionDown(InputNr: LongInt; var ICounter: LongInt;
            IPosition, TermInputNr: LongInt): LongInt;
function  ftiWaitForPositionUp(InputNr: LongInt; var ICounter: LongInt;
            IPosition, TermInputNr: LongInt): LongInt;
procedure ftiWaitForTime(mSek: LongInt);
procedure ftiPause(mSek: LongInt);

function  ftiFinish(InputNr: LongInt): Boolean;

implementation

uses
  Windows, Messages, SysUtils, Forms, Dialogs;
const
  EMaske: array[1..16] of LongInt= ($1,    $2,    $4,    $8,
                                    $10,   $20,   $40,   $80,
                                    $100,  $200,  $400,  $800,
                                    $1000, $2000, $4000, $8000);
  vbTrue = -1; vbFalse = 0;
var
  pftiDCB: TpftiDCB = @ftiDCB;

function umOpenInterface(pftiDCB: TpftiDCB; PortName: string): LongInt;
         stdcall; external 'umFish20.DLL';
function umCloseInterface(pftiDCB: TpftiDCB): LongInt;
         stdcall; external 'umFish20.DLL';
function umGetVersion(): LongInt;
         stdcall; external 'umFish20.DLL';
function umGetInput(pftiDCB: TpftiDCB; InputNr: LongInt): LongInt;
         stdcall; external 'umFish20.DLL';
function umSetMotor(pftiDCB: TpftiDCB; MotorNr, Direction: LongInt): LongInt;
         stdcall; external 'umFish20.DLL';
function umSetLamp(pftiDCB: TpftiDCB; LampNr, OnOff: LongInt): LongInt;
         stdcall; external 'umFish20.DLL';

function ftiOpenInterface(PortName: string): LongInt;
// --- Verbindung zu PortName herstellen ------------
begin
  ftiNotHalt := false;
  Result := umOpenInterface(pftiDCB, PortName);
end;

function ftiCloseInterface: LongInt;
// --- Verbindung zum Interface beenden -------------
begin
  Result := umCloseInterface(pftiDCB);
end;

function ftiGetInputs: LongInt;
// --- Auslesen des Status aller Digitaleingnge ----
begin
  Result := ftiDCB.InputStatus;
  Application.ProcessMessages;
end;

function ftiGetInput(InputNr: LongInt): Boolean;
// --- Auslesen Status Digitaleingang InputNr -------
begin
  Result := (umGetInput(pftiDCB, InputNr) = vbTrue);
  Application.ProcessMessages;
end;

function ftiGetAnalog(AnalogNr: LongInt): LongInt;
// --- Auslesen Wert Analogeingang AnalogNr ---------
begin
  Result := ftiDCB.Analogs[AnalogNr];
  Application.ProcessMessages;
end;

function ftiSetMotors(MotorStatus: LongInt): LongInt;
// --- Setzen aller Ausgnge ------------------------
begin
  ftiDCB.OutputStatus := MotorStatus;
  Result := ftiDCB.InputStatus;
end;

function ftiSetMotor(MotorNr, Direction: LongInt): LongInt;
// --- Setzen Ausgang MotorNr auf Direction ---------------
begin
  Result := umSetMotor(pftiDCB, MotorNr, Direction);
end;

function ftiClearMotors: LongInt;
// --- Ausschalten aller Ausgnge -------------------------
begin
  ftiDCB.OutputStatus := 0;
  Result := ftiDCB.InputStatus;
end;

function ftiSetLamp(LampNr, OnOff: LongInt): LongInt;
// --- Setzen eines "halben" Ausganges LampNr auf OnOff ----
begin
  Result := umSetLamp(pftiDCB, LampNr, OnOff);
end;

function ftiWaitForInput(InputNr: LongInt; OnOff: Boolean): LongInt;
// --- Warten bis Eingang = OnOff wird --------------
//     Abbruch bei ftiNotHalt, Escape

begin
  repeat
    Application.ProcessMessages;       // --- Unterbrechbarkeit
    if ftiNotHalt OR (GetAsyncKeyState(VK_ESCAPE) <> 0) then
    begin                              // --- Abbrechbarkeit
      Result := ftiFehler;
      exit;
    end;
  until (umGetInput(pftiDCB, InputNr) = vbTrue) = OnOff;
  Result := ftiDCB.InputStatus;
end;

function ftiWaitForChange(InputNr, NrOfChanges, TermInputNr: LongInt): LongInt;
// --- Warten auf eine oder mehrere (NrOfChanges) nderungen des Status
//     des Einganges InputNr bzw. TermInputNr = true ----------------------
//     Abbruch bei ftiNotHalt, Escape, Fehler, Endtaster

var M3: LongInt;
begin
  Result := ftiFehler;
  ftiDCB.Counters[InputNr] := 0;

  if TermInputNr = 0 then M3 := 0 else M3 := EMaske[TermInputNr];

  while ftiDCB.Counters[InputNr] < NrOfChanges do begin
    if ftiNotHalt OR (GetAsyncKeyState(VK_ESCAPE) <> 0) OR
       (ftiDCB.InputStatus = ftiFehler) OR
       ((ftiDCB.InputStatus AND M3) > 0) then exit;
    Application.ProcessMessages;
    end;
  Result := ftiDCB.InputStatus;
end;

function  ftiWaitForHigh(InputNr: LongInt): LongInt;
// --- Warten auf einen false/true Durchgang von InputNr
//     Abbruch bei ftiNotHalt, Escape, Fehler
begin
  ftiWaitForInput(InputNr, false);
  Result := ftiWaitForInput(InputNr, true);
end;

function ftiWaitForLow(InputNr: LongInt): LongInt;
// --- Warten auf einen true/false Durchgang von InputNr ----
//     Abbruch bei ftiNotHalt, Escape, Fehler
begin
  ftiWaitForInput(InputNr, true);
  Result := ftiWaitForInput(InputNr, false);
end;

function  ftiWaitForPositionDown(InputNr: LongInt; var ICounter: LongInt;
            IPosition, TermInputNr: LongInt): LongInt;
// --- Warten auf IPosition = ICounter durch Subtrahieren der Impulse
//     des Einganges InputNr bzw. TermInputNr = true
//     Abbruch bei ftiNotHalt, Escape, Fehler, Endtaster
begin
  Result := ftiWaitForChange(InputNr, ICounter - IPosition, TermInputNr);
  ICounter := ICounter - ftiDCB.Counters[InputNr];
end;

function  ftiWaitForPositionUp(InputNr: LongInt; var ICounter: LongInt;
            IPosition, TermInputNr: LongInt): LongInt;
// --- Warten auf IPosition = ICounter durch Addieren der Impulse
//     des Einganges InputNr bzw. TermInputNr = true
//     Abbruch bei ftiNotHalt, Escape, Fehler, EndTaster
begin
  Result := ftiWaitForChange(InputNr, IPosition - ICounter, TermInputNr);
  ICounter := ICounter + ftiDCB.Counters[InputNr];
end;

procedure ftiWaitForTime(mSek: LongInt);
// --- Eine vorgegebene Zeit warten --------------
//     Abbruch bei ftiNotHalt, Escape

var sZeit, nSek: DWORD;

begin
  nSek  := mSek;
  sZeit := GetTickCount();
  repeat
    Application.ProcessMessages;     // --- Unterbrechbarkeit
  until ftiNotHalt OR (GetTickCount() - sZeit > nSek)
                OR (GetAsyncKeyState(VK_ESCAPE)<>0);
end;

procedure ftiPause(mSek: LongInt);
// --- Eine vorgegebene Zeit warten -----------------
//     Abbruch bei ftiNotHalt, Escape
//     Synonym von ftiWaitForTime (aus Kompatibilittsgrnden)
begin
  ftiWaitForTime(mSek);
end;

function ftiFinish(InputNr: LongInt): Boolean;
// --- Feststellen eines Endewunsches : ftiNotHalt, Escape, InputNr ----
begin
  Result := false;
  if ftiNotHalt OR (GetAsyncKeyState(VK_ESCAPE) <> 0) then Result := true;
  if InputNr > 0 then if (umGetInput(pftiDCB, InputNr) = vbTrue) then
                          Result := true;
  Application.ProcessMessages;
end;

end.
